import requests
import re
from Bio import Phylo, SeqIO
from io import StringIO


def family_seq(families, form='fasta', type='full', download = False, path = None):
    """This function search for sequences of the family in format and form specified by the user.

    :param families: Id or accession of the family, or list of the families names
    :type families: str or list
    :param form: Output format of ssequences. Available: fasta, selex, stockholm, msf, defaults to fasta
    :type form: str, optional
    :param type: A type of the sequences from the fmaily site. Available: full, seed, defaults to full
    :type type: str, optional
    :param download: 'True' if download to the file, defaults to False
    :type download: bool optional
    :param path: path to the outpur file if download is True
    :type path: str, optional
    :return: List of SeqIO type objects
    :rtype: list
    """
    result = []
    forms = ['fasta', 'selex', 'stockholm', 'msf']
    types = ['seed', 'full']
    if form not in forms:
        raise ValueError("form can be one of following: fasta, selex, stockholm, msf")
    if type not in types:
        raise ValueError("type can be one of following: seed, full")
    if not isinstance(families, (list, str)):
        raise ValueError("incorrect type of families name")
    if isinstance(families, str):
        families = [families]
    for family in families:
        print('Now downloading: %s' %family)
        url = 'https://pfam.xfam.org/family/%s' % family
        url += '/alignment/%s' % type
        url += '/format?format=%s&alnType=%s&order=a&case=l&gaps=dashes&download=1' % (form, form)
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        if form != 'selex':
            result.append(SeqIO.parse(StringIO(r.text), form))
        else:
            result.append(r.text)
        if download:
            if path is not None:
                pathname = '%s/%s.%s' % (path, family, form)
            else:
                pathname = '%s.%s' % (family, form)
            f = open(pathname, 'w').write(r.text)
    return result

def clan_members(clans):
    """This function return the dictionary of pfam families belonging to the clans
    
    :param clans: list or string with the name of the clan
    :type path: list/str
    :return: Ditionary of clan name as key and clan members as value
    :rtype: dict

    """
    result = {}
    if not isinstance(clans, (list, str)):
        raise ValueError("incorrect type of clan name")
    if isinstance(clans, str):
        clans = [clans]
    for clan in clans:
        clan_url = 'https://pfam.xfam.org/clan/%s#tabview=tab2' % clan
        r_clan = requests.get(clan_url, allow_redirects=True)
        r_clan.raise_for_status()
        families = re.findall('PF[0-9]{5}',r_clan.text)
        result[clan] = families
    return result

def family_tree(families, download=False, path=None):
    """This function search for phylogenetic tree of the family in the Newick format.

    :param families: Id or accession of the family, or list of the families names
    :type families: str or list
    :param download: 'True' if download to the file, defaults to False
    :type download: bool optional
    :param path: path to the outpur file if download is True
    :type path: str, optional
    :return: List of PhyloIO type objects
    :rtype: list
    """
    result = []
    if not isinstance(families, (list, str)):
        raise ValueError("incorrect type of families name")
    if isinstance(families, str):
        families = [families]
    for family in families:
        url = 'https://pfam.xfam.org/family/%s/tree/download' % family
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        tree = Phylo.read(StringIO(r.text), "newick")
        result.append(tree)
        if download:
            if path is not None:
                pathname = '%s/%s.nhx' % (path, family)
            else:
                pathname = '%s.nhx' % (family)
            f = open(pathname, 'w').write(r.text)
    return result


def get_alternative(families):
    """This function give alternative name for the family - for id it gves accession,
    for accession - gives id.

    :param families: Id or accession of the family
    :type families: str or list
    :return: Dictionary with the accession as key and id as value
    :rtype: dict
    """
    results = {}
    if not isinstance(families, (list, str)):
        raise ValueError("incorrect type of families name")
    if isinstance(families, str):
        families = [families]
    for family in families:
        url = 'https://pfam.xfam.org/family/%s' % family
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        titles = re.search('<h1>Family:.+?</h1>', r.text).group()
        acc = titles[titles.find("(")+1:titles.find(")")]
        id_pfam = titles[titles.find("<em>")+4:titles.find("</em>")]
        if family == acc:
            results[acc] = id_pfam
        else:
            results[id_pfam] = acc
    return results

def go_terms(family):
    """This function give go terms connected to the pfam family

    :param families: Id or accession of the family
    :type families: str
    :return: Set of go terms
    :rtype: set
    """
    url = 'https://pfam.xfam.org/family/%s' % family
    r = requests.get(url, allow_redirects=True)
    r.raise_for_status()
    gos = set(re.findall('GO:[0-9]+', r.text))
    return gos



from .xfam_object import *
from .xfam_to import rfam_clan_to_pdb

class RfamClan(XfamObject):
    """conteins inforation about Rfam Clan.
    
    :param access: Rfam access of clan
    :type access: str
    :param short_name: Rfam id of clan
    :type short_name: str
    :param type: type of rfam entry
    :type type: str
    :param pdb-ref: List of PDB names associated with a family
    :type pdb_ref: list
    :param members: List of families that are members of this clan
    :type members: list
    """
    def __init__(self,clan):
        """Constructor method.
        
        :param clan: rfam access or clan id
        :type family: str
        """
        super().__init__(clan,db='rfam',type_='clan')
        self.type='clan'
        self.pdb_ref=rfam_clan_to_pdb(self.access)
        self.members=self.__set_members()
        
    def __set_members(self):
        """Method for setting members attribute.
        
        :return: List of clan members
        :rettype: list"""
        
        clan=self.access
        clan_url = 'https://rfam.xfam.org/clan/%s#tabview=tab2' % clan
        r_clan = requests.get(clan_url, allow_redirects=True)
        r_clan.raise_for_status()
        families = re.findall('RF[0-9]{5}',r_clan.text)
        return list(set(families))




import requests
import re
from Bio import SeqIO
from io import StringIO

def rfamily_seq(families, form='fasta', download = False, path = None):
    """This function search for sequences of the family in format and form specified by the user.
    
    :param families: accession of the family, or list of the families accession
    :type families: str or list
    :param form: Output format of ssequences. Available: fasta, stockholm, defaults to fasta
    :type form: str, optional
    :param download: 'True' if download to the file, defaults to False
    :type download: bool optional
    :param path: path to the outpur file if download is True
    :type path: str, optional
    :return: List of SeqIO type objects
    :rtype: list
    """
    result = []
    forms = ['fasta', 'stockholm']
    if form not in forms:
        raise ValueError("form can be one of following: fasta, selex,stockholm")
    if not isinstance(families, (list, str)):
        raise ValueError("incorrect type of families name")
    if isinstance(families, str):
        families = [families]
    for family in families:
        print('Now downloading: %s' %family)
        url = 'https://rfam.xfam.org/family/%s' % family
        url += '/alignment?acc=%s' % family
        url += '&format=%s&download=0' % form
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        result.append(SeqIO.parse(StringIO(r.text), form))
        if download:
            if path is not None:
                pathname = '%s/%s.%s' % (path, family, form)
            else:
                pathname = '%s.%s' % (family, form)
            f = open(pathname, 'w').write(r.text)
    return result



#__init__py

from .family_architectures import *
from .hammer_to_object import*
from .hmm_autosearch import*
from .hmm_download import*
from .hmmer_command import*
from .hmmer_file import*
from .pfam_architecture import*
from .pfam_clan import*
from .pfam_family import*
from .pfam_sequences import*
from .rfam_clan import*
from .rfam_family import*
from .rfamily_seq import*
from .xfam_object import*
from .xfam_to import*



"""
module in which is implemented one class - HMMERProfileFileBuilder which enables to parse HMMER profile file
"""
class HMMERProfileFileBuilder:
  """
  class for parsing HMMER profile file

  enables to calculate length of profile
  adding new positions of profile
  printing out object
  saving object to a functioning HMMER file which could be used for analisys by HMMER tool
  """
  def __init__(self, version, name, acc, desc, alph, rf, mm, cons, cs, map, date, nseq, effn,
               cksum, ga, tc, nc, bm, sm, stats_local_msv, stats_local_viterbi, stats_local_forward, match_emission, insert_emission, state_transition):
    """class for parsing HMMER profile file
      This class uses following arguments:

    :param version: version of hmm tool
    :type version: str
    :param name: name of protein
    :type name: str
    :param acc: accession number
    :type acc: str
    :param desc: short description of protein
    :param alph: type of alphabet
    :type alph: str
    :param rf: Reference annotation flag
    :type rf: bool
    :param mm: Model masked flag
    :type mm: bool
    :param cons: Consensus residue annotation flag
    :type cons: bool
    :param cs: Consensus structure annotation flag
    :type cs: bool
    :param map: Map annotation flag
    :type map: bool
    :param date: Date the model was constructed
    :type data: str
    :param nseq: Sequence number
    :type nseq: int
    :param effn: Effective sequence number
    :type effn: float
    :param cksum: Training alignment checksum
    :type cksum: int
    :param ga: Pfam gathering thresholds GA1 and GA2. See Pfam documentation of GA lines
    :type ga: list
    :param tc: Pfam trusted cutoffs TC1 and TC2. See Pfam documentation of TC lines
    :type tc: list
    :param nc: Pfam noise cutoffs NC1 and NC2. See Pfam documentation of NC lines
    :type nc: list
    :param bm: hmmbuild command
    :type bm: str
    :param sm: hmmsearch command
    :type sm: str
    :param stats_local_msv: statistics of local msv
    :type stats_local_msv: list
    :param stats_local_viterbi: statistics of local viterbi
    :type stats_local_viterbi: list
    :param stats_local_forward: statistics of local forward:
    :type stats_local_forward: list
    :param match_emission: dictionary of -log0.25 probabilities of match emission
    :type match_emission: dict
    :param insert_emission: dictionary of -log0.25 probabilities of insert emission
    :type insert_emission: dict
    :param state_transition: dictionary of -log0.25 probabilities of state transitions
    :type state_transition: dict
    """

    self.version = version
    self.name = name
    self.acc = acc
    self.desc = desc
    self.alph = alph
    self.rf = rf
    self.mm = mm
    self.cons = cons
    self.cs = cs
    self.map = map
    self.date = date
    self.nseq = nseq
    self.effn = effn
    self.cksum = cksum
    self.ga = ga
    self.tc = tc
    self.nc = nc
    self.bm = bm
    self.sm = sm
    self.stats_local_msv = stats_local_msv
    self.stats_local_viterbi = stats_local_viterbi
    self.stats_local_forward = stats_local_forward
    self.profile = {"match_emission" : match_emission, "insert_emission" : insert_emission, "state_transition" : state_transition}

  def get_length(self):
    """
    return length of profile
    :return: length of profile
    :rtype: int
    """
    self.leng = len(self.profile["match_emission"])
    return self.leng

  def add_position(self, match_emission, insert_emission, state_transition):
    """
    adding match_emission probabilities, insert_emission probabilities, state_transition probabilities of new position

    :param match_emission: dictionary of -log0.25 probabilities of match emission
    :type match_emission: dict
    :param insert_emission: dictionary of -log0.25 probabilities of insert emission
    :type insert_emission: dict
    :param state_transition: dictionary of -log0.25 probabilities of state transition
    :type state_transition: dict
    """
    self.profile["match_emission"] += [match_emission]
    self.profile["insert_emission"] += [insert_emission]
    self.profile["state_transition"] += [state_transition]

  def __str__(self):
    """
    print out object

    :return: string representation of objct
    :rtype: str
    """
    list_match = self.profile["match_emission"]
    list_insert = self.profile["insert_emission"]
    list_transition = self.profile["state_transition"]
    length_model = self.get_length()
    model_str="M - match emmision I - insert emmision T - state transition length =" + str(length_model) + "\n"
    for i in range(length_model):
      model_str += str(i) + '\t' + 'M' + '\t' + str(list_match[i]) + '\n' + '\t' + 'I' + '\t' +  str(list_insert[i]) + '\n' + '\t' + 'T' + '\t' + str(list_transition[i]) + '\n'
    return model_str.strip()

  def file_format(self, filename):
    """
    save object to HMMER profile file which could be used for calculation by hmmsearch and hmmscan

    :param filename: name of new hmm file which will be created
    :return: hmm file
    :rtype: file
    """
    file = open(filename, "w")
    length_model = self.get_length() - 1
    file_text = self.version + '\n'
    file_text += "NAME" + ' ' + self.name + '\n' + "ACC" + ' ' + self.acc + '\n' + "DESC"+ ' ' + self.desc + '\n' + "LENG" + ' ' + str(length_model) +'\n' \
                 + "ALPH" +  ' ' + self.alph + '\n'
    if self.rf == True:
      file_text += "RF" + ' ' + "yes" + '\n'
    else:
      file_text += "RF" + ' ' + "no" + '\n'
    if self.mm == True:
      file_text += "MM" + ' ' + "yes" + '\n'
    else:
      file_text += "MM" + ' ' + "no" + '\n'
    if self.cons == True:
      print("CONS", self.cons)
      file_text += "CONS" + ' ' + "yes" + '\n'
    else:
      file_text += "CONS" + ' ' + "no" + '\n'
    if self.cs == True:
      file_text += "CS" + ' ' + "yes" + '\n'
    else:
      file_text += "CS" + ' ' + "no" + '\n'
    if self.map == True:
      file_text += "MAP" + ' ' + "yes" + '\n'
    else:
      file_text += "MAP" + ' ' + "no" + '\n'
    file_text += "DATE" + ' ' + str(self.date) + '\n' + "NSEQ" + ' ' + str(self.nseq) + '\n'+ "EFFN" + ' ' + str(self.effn) + '\n' + "CKSUM" +  ' ' + str(self.cksum) + '\n'
    file_text += "GA" + ' ' + str(self.ga[0]) + ' ' + str(self.ga[1]) + ";"  + '\n' + "TC" + ' ' + str(self.tc[0]) + ' ' + str(self.tc[1]) + ";" + '\n' + "NC" + ' ' \
                 + str(self.nc[0]) + ' ' + str(self.nc[1]) + ";" + '\n'
    file_text += "BM" + ' ' + str(self.bm) + '\n' + "SM" + ' ' + str(self.sm) + '\n'
    file_text += "STATS LOCAL MSV" + ' ' + str(self.stats_local_msv[0]) + ' ' + str(self.stats_local_msv[1]) + '\n' + "STATS LOCAL VITERBI" + ' ' + str(self.stats_local_viterbi[0]) \
                 + ' ' + str(self.stats_local_viterbi[1]) +'\n'+ "STATS LOCAL FORWARD" + ' ' + str(self.stats_local_forward[0]) + ' ' + str(self.stats_local_forward[1]) +'\n'
    file_text += "HMM          A        C        D        E        F        G        H        I        K        L        M        N        P        Q        R        S        T        V        W        Y"+'\n'  \
                 +"\t\t\tm->m     m->i     m->d     i->m     i->i     d->m     d->d\n"
    match = self.profile["match_emission"]
    emission = self.profile["insert_emission"]
    transition = self.profile["state_transition"]
    file_text += "  COMPO   "
    for i in range(length_model +1):
      if i !=0:
        file_text += " " * (7-len(str(i))) + str(i) + "   "
        for m in match[i]:
          file_text += str(format(match[i][str(m)],".5f"))  + "  "
        file_text= file_text.strip("  ")
        file_text += " " * (7-len(str(i))) + str(i) + " k - - -" + "\n" + "          "
        for e in emission[i]:
          file_text += str(format(emission[i][str(e)],".5f")) + "  "
        file_text = file_text.strip("  ")
        file_text +="\n" + "          "
        for t in transition[i]:
          elt = transition[i][t]
          if elt is None:
            file_text += "      " + "*  "
          else:
            file_text += str(format(elt,".5f")) + "  "
        file_text = file_text.strip("  ")
        file_text += "\n"
      else:
        for m in match[i]:
          file_text += str(format(match[i][str(m)],".5f")) + "  "
        file_text = file_text.strip("  ")
        file_text += "\n" + "          "
        for e in emission[i]:
          file_text += str(format(emission[i][str(e)],".5f")) + "  "
        file_text = file_text.strip("  ")
        file_text += "\n" + "          "
        for t in transition[i]:
          elt = transition[i][t]
          if elt is None:
            file_text += "      " + "*  "
          else:
            file_text += str(format(elt,".5f")) + "  "
        file_text = file_text.strip("  ")
        file_text += "\n"
    file_text +="//"
    file.write(file_text)



import requests
import re
from urllib.request import urlopen, Request
from urllib.error import HTTPError, URLError
from bs4 import BeautifulSoup

def _xfam_to(url, pattern, sep='/'):
    """serch for href pttern in given url respone, should not be used be users.
    
    :param url: url
    :type url: str
    :param pattern: pattern you are looking for
    :type pattern: str"""
    try:
        html=urlopen(url)
    except (HTTPError, URLError) as e:
        print(e, url)
        return None
    else:
        soup=BeautifulSoup(html, 'html.parser')
        names_set=set()
        for a in soup.find_all('a', href=True):
            if bool(re.match(pattern, a['href'])):
                names_set.add(a['href'].split(sep)[-1])
        return list(names_set)

        
def rfam_to_pubmed(family):
    """get a list of associated pubmed ids for given rfam access key.
    
    :param family: rfam accession key of family
    :type family: str
    :return: List of associated pubmed ids
    :rettype:list
    """
    url ='https://rfam.xfam.org/family/'+family
    pattern='http://www.ncbi.nlm.nih.gov/pubmed/'
    return ["".join(filter(str.isdigit, x)) for x in _xfam_to(url, pattern)]

def rfam_to_go(family):
    
    """get a list of associated GO ids for given rfam access key.
    
    :param family: rfam accession key of family
    :type family: str
    :return: List of associated Go ids
    :rettype:list"""
    url ='https://rfam.xfam.org/family/'+family
    pattern='https://www.ebi.ac.uk/QuickGO/term/'
    return _xfam_to(url, pattern)

def rfam_to_so(family):
    """ get a list of associated SO ids for given rfam access key.
    
    :param family: rfam accession key of family
    :type family: str
    :return: List of associated SO ids
    :rettype:list"""
    url ='https://rfam.xfam.org/family/'+family
    pattern='http://www.sequenceontology.org/miso/'
    return _xfam_to(url, pattern)


def rfam_to_pdb(family):
    """get a list of associated PDB ids for given rfam access key.
    
    :param family: rfam accession key of family
    :type family: str
    :return: List of associated PDB ids
    :rettype:list"""
    url ='https://rfam.xfam.org/family/'+family+'/structures'
    pattern='http://www.rcsb.org/pdb/'
    return [i.upper() for i in _xfam_to(url, pattern, sep='=')]
    
def pfam_to_pdb(family):
    """get a list of associated PDB ids for given pfam access key.
    
    :param family: pfam accession key of family
    :type family: str
    :return: List of associated PDB ids
    :rettype:list"""
    url ='https://pfam.xfam.org/family/'+family+'/mapping'
    pattern='https://www.ebi.ac.uk/interpro/structure/PDB/'
    return _xfam_to(url,pattern)

def pfam_to_so(family):
    """get a list of associated SO ids for given pfam access key.
    
    :param family: pfam accession key of family
    :type family: str
    :return: List of associated SO ids
    :rettype:list"""
    url ='https://pfam.xfam.org/family/'+family
    pattern='http://www.sequenceontology.org/miso/'
    return _xfam_to(url, pattern)

def pfam_to_pubmed(family):
    """get a list of associated pubmed ids for given pfam access key.
    
    :param family: pfam accession key of family
    :type family: str
    :return: List of associated Pubmed ids
    :rettype:list"""
    url='https://pfam.xfam.org/family/'+family
    pattern='http://www.ncbi.nlm.nih.gov/pubmed/'
    return _xfam_to(url,pattern)

def pfam_to_go(family):
    """get a list of associated GO ids for given pfam access key.
    
    :param family: pfam accession key of family
    :type family: str
    :return: List of associated GO ids
    :rettype:list"""
    url ='https://pfam.xfam.org/family/'+family
    pattern='http://www.ebi.ac.uk/ego/'
    return _xfam_to(url, pattern,sep='=')

def pfam_clan_to_pdb(clan):
    """get a list of associated PDB ids for given pfam clan access key.
    
    :param clan: pfam accession key of clan
    :type clan: str
    :return: List of associated PDB ids
    :rettype:list"""
    url='http://pfam.xfam.org/clan/'+clan+'/structures'
    pattern='/structure/[A-Z, 0-9]{4}'
    return _xfam_to(url,pattern)

def rfam_clan_to_pdb(clan):
    """get a list of associated PDB ids for given rfam clan access key.
    
    :param clan: rfam accession key of clan
    :type clan: str
    :return: List of associated PDB ids
    :rettype:list"""
    url ='https://rfam.xfam.org/clan/'+clan+'/structures'
    pattern='http://www.rcsb.org/pdb/'
    return [i.upper() for i in _xfam_to(url, pattern, sep='=')]



import requests
import re
from Bio import Phylo, SeqIO
from io import StringIO
from bs4 import BeautifulSoup
from urllib.request import urlopen

class XfamObject():
    """An abstract class, base for PfamFamily,PfamClan,RfamFamily and RfamClan."""
    
    def __init__(self,name,db,type_):
        """constructor method.
        :param name: rfam/pfam entry access or id
        :type name: str
        :param db: database, allowed 'pfam' or 'rfam'
        :type db: str
        :param type_: type of entry, allowed 'family' or 'clan'
        :type type_: str
        """
        self.db=db
        self.access = self.__set_access(db, type_, name)
        self.short_name = self.__set_id(db, type_, name)
        if self.access is None and self.short_name is None:
            raise ValueError()
    
    def __set_access(self, db,type_,name):
        """get access key for a given name.
        
        :param name: access or id of entry
        :type name: str
        :param type_: type of entry - clan or family
        :type type_: str
        :return: access key
        :rettype: str
        """
        url = 'https://%s.xfam.org/%s/%s' % (db,type_,name)
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        titles = re.search('<h1>%s:.+?</h1>'%type_.capitalize(), r.text).group()
        acc = titles[titles.find("(")+1:titles.find(")")]
        return acc
    def __set_id(self, db, type_, name):
        """get id for a given name.
        
        :param name: access or id of entry
        :type name: str
        :param type_: type of entry - clan or family
        :type type_: str
        :return: entry id
        :rettype: str
        """
        url = 'https://%s.xfam.org/%s/%s' % (db,type_,name)
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        titles = re.search('<h1>%s:.+?</h1>'%type_.capitalize(), r.text).group()
        id_pfam = titles[titles.find("<em>")+4:titles.find("</em>")]
        return id_pfam
    def __str__(self):
        rep = self.db.capitalize()+' ' + self.type + ' ' + self.short_name + ' '
        rep += '; ' + self.db.capitalize()+' Access: '
        rep += self.access
        return rep
    
    def get_tree(self):
        """Get phylogenetic tree of entry if entry is not a clan.
        
        :return: Phylogenetic tree in newick format 
        :rettype: Bio.Phylo.Newick.Tree
        """
        if self.type=='clan':
            return None
        url = 'https://%s.xfam.org/family/%s/tree/' % (self.db,self.access)
        if self.db=='pfam':
            url+='download'
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        tree = Phylo.read(StringIO(r.text), "newick")
        return tree



from .xfam_object import *
from .xfam_to import *

class RfamFamily(XfamObject):
    """contains information about Rfam familly.
    
    :param access: Rfam access of family
    :type access: str
    :param short_name: Rfam id of family
    :type short_name: str
    :param type: type of rfam entry
    :type type: str
    :param go_ref: List of GO ids associated with a family
    :type go_ref: list
    :param so_ref: List of SO ids associated with a family
    :type so_ref: list
    :param pubmed_ref: List of Pubmed ids associated with a family
    :type pubmed_ref: list
    :param pdb_ref: List of PDB names associated with a family
    :type pdb_ref: list
    """
    def __init__(self,family):
        """Constructor method.
        
        :param family: rfam access or family id
        :type family: str
        """
        super().__init__(family,db='rfam',type_='family')
        self.type='family'
        self.go_ref=rfam_to_go(self.access)
        self.so_ref=rfam_to_so(self.access)
        self.pubmed_ref=rfam_to_pubmed(self.access)
        self.pdb_ref=rfam_to_pdb(self.access)
        
    def get_sequences(self):
        """Get alignemts of RfamFamily sequences in fasta format.
        
        :return: Biopython iterator of sequences from alignment  
        :rtype: Bio.SeqIO.FastaIO.FastaIterator"""
        url = 'https://rfam.xfam.org/family/%s' % self.access
        url += '/alignment?acc=%s' % self.access
        url += '&format=fastas&download=0'
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        return SeqIO.parse(StringIO(r.text), 'fasta')
    



from .xfam_object import *
from .family_architectures import *
from .xfam_to import *

class PfamClan(XfamObject):
    """conteins information about Pfam clan.
    
    :param access:
    :type access: str
    :param short_name:
    :type short_name:str
    :param type: type of pfam entry
    :type type: str
    :param scop_id: SCOP identificator of clan
    :type scop_id: str
    :param description: Short description of clan
    :type description: str
    :param pdb_ref: List of PDB names associated with a family
    :type pdb_ref: list
    :param members: List of families that are members of this clan
    :type members: list
    """
    def __init__(self,clan):
        """Constructor method.
        :param clan: pfam access or clan id
        :type clan: str
        """
        super().__init__(clan,db='pfam', type_='clan')
        self.type='clan'
        url='http://pfam.xfam.org/clan/browse'
        try:
            html=urlopen(url)
        except (HTTPError, URLError) as e:
            print(e, url)
            return None
        else:
            soup=BeautifulSoup(html, 'html.parser')
            table=soup.find('table',attrs={"class": "details browse"})
            for a in table.find_all('a',href=True):
                if a.string==self.access:
                    tr=a.parent.parent
                    columns=tr.find_all('td')
                    if columns[2].span:
                        self.scop_id=None
                    else:
                        self.scop_id="".join(filter(str.isdigit, columns[2].a.string))
                    self.description=columns[3].string
        self.pdb_ref=pfam_clan_to_pdb(self.access)
        self.members=self.__set_members()
        
    def __set_members(self):
        """Method for setting members attribute.
        
        :return: List of clan members
        :rettype: list"""
        clan=self.access
        clan_url = 'https://pfam.xfam.org/clan/%s#tabview=tab2' % clan
        r_clan = requests.get(clan_url, allow_redirects=True)
        r_clan.raise_for_status()
        families = re.findall('PF[0-9]{5}',r_clan.text)
        return list(set(families))
        
    def get_architectures(self):
        """Get a list of architectures of Pfam clan.
        
        :return: list of PfamArchitecture objects
        :rettype: list
        """
        return family_architectures(self.access,clan=True)
